import { BehaviorSubject } from 'rxjs';

var FetchStatus;
(function(FetchStatus) {
  FetchStatus[(FetchStatus['ready'] = 0)] = 'ready';
  FetchStatus[(FetchStatus['fetching'] = 1)] = 'fetching';
  FetchStatus[(FetchStatus['finish'] = 2)] = 'finish';
})(FetchStatus || (FetchStatus = {}));
class FlyEnumSelectService {
  constructor() {
    this.enumInfoMap = new Map();
    this.url = '';
    this.axios = null
  }
  static getInstance() {
    if (
      !FlyEnumSelectService.instance ||
      !(FlyEnumSelectService.instance instanceof this)
    ) {
      FlyEnumSelectService.instance = new this();
    }
    return FlyEnumSelectService.instance;
  }

  fetch(url, params, reload = false, axios) {
    if (!this.axios) {
      this.axios = axios
    }
    this.url = url;
    let fetcher;
    // 从缓存里面拿字典
    let enumInfo = this.enumInfoMap.get(params.code);
    if (enumInfo && !reload) {
      fetcher = enumInfo.fetcher;
    } else {
      // 缓存里面没有东西，就创建一个
      fetcher = new BehaviorSubject(false);
      this.enumInfoMap.set(params.code, {
        code: params.code,
        fetcher: fetcher,
        status: FetchStatus.ready,
      });
    }
    // 函数节流
    clearTimeout(this._debounceId);
    this._debounceId = setTimeout(() => {
      // 获取没有发送过请求的字典
      let codes = [];
      for (let enumInfo of Array.from(this.enumInfoMap.values())) {
        if (enumInfo.status == FetchStatus.ready) {
          enumInfo.status = FetchStatus.fetching;
          codes.push(enumInfo.code);
        }
      }
      if (codes.length < 1) {
        return;
      }
      if (!this.axios) {
        console.error('请传入axios!')
        return
      }
      this.axios.post(this.url, {
        codes: codes,
      }).then(result => {
        let map = result.data;
        Object.keys(map).forEach(key => {
          let result = map[key];
          let enumInfo = this.enumInfoMap.get(key);
          enumInfo.result = result;
          enumInfo.fetcher.next({ data: result });
          enumInfo.status = FetchStatus.finish;
        });
      });
    }, 50);
    return fetcher;
  }
  reduceFetcher() {
    return true;
  }
}
export default FlyEnumSelectService;
