import {BehaviorSubject} from 'rxjs';
import isEqual from './is-equal';

var FetchStatus;
(function (FetchStatus) {
  FetchStatus[(FetchStatus['ready'] = 0)] = 'ready';
  FetchStatus[(FetchStatus['fetching'] = 1)] = 'fetching';
  FetchStatus[(FetchStatus['finish'] = 2)] = 'finish';
})(FetchStatus || (FetchStatus = {}));

class FlyPickerService {
  constructor() {
    //根据id进行缓存
    // private _caches:Map<string,CacheInfo> = new Map();
    //等待列表
    // private _waitForData:Map<Array<string>,BehaviorSubject<any>> = new Map();
    //函数节流用的id
    // private _clearCacheId:any;
    //以链接做key，请求信息的数组作为value
    this.fetchInfosMap = new Map();
    //用请求保存函数节流
    this._clearCacheIdMap = new Map();
    //用请求保存缓存
    this._cachesMap = new Map();
    //用请求保存函数节流
    this._waitForDataMap = new Map();
    this.axios = null
  }

  fetchObjectById(key, ids, url, axios) {
    if (!this.axios) {
      this.axios = axios
    }
    let fetcher = new BehaviorSubject(false);
    let waitForData = this._waitForDataMap.get(url) || new Map();
    waitForData.set(ids, fetcher);
    this._waitForDataMap.set(url, waitForData);
    let caches = this._cachesMap.get(url) || new Map();
    //分解所有的id，然后每个都带上状态
    ids.forEach(id => {
      caches.set(id, {id: id, status: FetchStatus.ready});
    });
    this._cachesMap.set(url, caches);
    //函数节流
    clearTimeout(this._clearCacheIdMap.get(url));
    let _clearCacheId = setTimeout(() => {
      //获取所有需要请求的id
      let allIds = Array.from(caches.values())
        .filter(info => info.status === FetchStatus.ready)
        .map(info => {
          info.status = FetchStatus.fetching;
          return info.id;
        });
      if (allIds.length > 0) {
        this._doFetchObjectById(key, allIds, url);
      }
    }, 30);
    this._clearCacheIdMap.set(url, _clearCacheId);
    return fetcher;
  }

  _doFetchObjectById(key, ids, url) {
    if (!this.axios) {
      console.error('请传入axios!')
      return
    }
    //发送请求
    this.axios.post(url, {
      criteria: {
        [key]: {
          in: ids
        }
      },
      page: 0,
      size: 2000
    }).subscribe(result => {
      let caches = this._cachesMap.get(url);
      let waitForData = this._waitForDataMap.get(url);
      //更改状态为完毕
      result.data.forEach(user => {
        let info = caches.get(user[key]);
        info.status = FetchStatus.finish;
        info.result = user;
      });
      //遍历等待发送数据的列表，从缓存里面拿数据，只要不完整就中断，完整就发送，然后从等待列表中移除
      waitForData.forEach((fetcher, ids) => {
        let result = [];
        for (let id of ids) {
          let info = caches.get(id);
          if (info && info.status == FetchStatus.finish) {
            result.push(info.result);
          } else {
            return;
          }
        }
        fetcher.next({data: result});
        waitForData.delete(ids);
        fetcher.complete();
      });
      //等待列表里面没内容了，就移除缓存
      if (waitForData.size == 0) {
        caches.clear();
      }
    });
  }

  /**
   * 发送请求咯
   */
  fetch(url, params, reload, axios) {
    if (!this.axios) {
      this.axios = axios
    }
    return this.getFetcher(url, params, reload);
  }

  /**
   * 获取请求器
   * @param reload 是否刷新数据, 从服务器获取
   */
  getFetcher(url, params, reload) {
    let fetchInfos = reload ? null : this.fetchInfosMap.get(url);
    //以前根本就没有创建过请求，就创建一个吧
    if (reload || !fetchInfos || fetchInfos.length == 0) {
      let fetcher = this.createFetcher(url, params);
      fetchInfos = [{params: params, fetchInfo: fetcher, count: 1}];
      this.fetchInfosMap.set(url, fetchInfos);
      return fetcher;
    }
    //在已有的数组里面找,找到就返回
    for (let fetchInfo of fetchInfos) {
      if (isEqual(fetchInfo.params, params)) {
        fetchInfo.count++;
        return fetchInfo.fetchInfo;
      }
    }
    //数组里面没有啊，那么就创建一个，然后返回
    let fetcher = this.createFetcher(url, params);
    fetchInfos.push({params: params, fetchInfo: fetcher, count: 1});
    return fetcher;
  }

  /**
   * 清空请求器相关的信息
   */
  reduceFetcher(url, params) {
    let fetchInfos = this.fetchInfosMap.get(url);
    if (fetchInfos && fetchInfos.length > 0) {
      //在已有的数组里面找,找到就返回
      for (let index = 0; index < fetchInfos.length; index++) {
        let fetchInfo = fetchInfos[index];
        if (isEqual(fetchInfo.params, params)) {
          fetchInfo.count--;
          if (fetchInfo.count == 0) {
            fetchInfos.splice(index, 1);
          }
          return true;
        }
      }
    }
    return false;
  }

  /**
   * 创建新的请求器
   */
  createFetcher(url, params) {
    const fetcher = new BehaviorSubject(false);
    if (!this.axios) {
      console.error('请传入axios!')
      return
    }
    this.axios.post(url, params).subscribe(result => {
      fetcher.next(result);
    });
    return fetcher;
  }
}

const entity = new FlyPickerService();
export default entity;
